/*
  kcontrol - Base for KDE Control Applications

  written 1997 by Matthias Hoelzer

    Modifications to this program were made by Corel Corporation,
	  November, 1999.  All such modifications are copyright (C) 1999
    Corel Corporation and are licensed under the terms of the LGPL License.
  		 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.
  */


#ifndef _K_CONTROL_
#define _K_CONTROL_

#include <unistd.h>
#include <errno.h>

#include <qsocketnotifier.h>
#include <qtabbar.h>
#include <qtabdialog.h>
#include <qstrlist.h>
#include <qpushbutton.h>
#include <qaccel.h>

#include <kapp.h>
#include <kprocess.h>

class KControlApplication;


/** KControlDialog is a QTabDialog that is internally used by KControl Applications.
*/
class KControlDialog : public QTabDialog
{
  Q_OBJECT

  friend KControlApplication;

public:

  /// Initializes the dialog
  KControlDialog();


// access methods to enable/disable show and hide buttons of the Control dialogs	(AHMEDM)
  void enableCancelButton( bool bEnable )		{ enableWidget( m_lppbCancelBtn, bEnable ); }
  void enableApplyButton( bool bEnable )		{ enableWidget( m_lppbApplyBtn, bEnable ); }
  void enableOkButton( bool bEnable )		{ enableWidget( m_lppbOkBtn, bEnable ); }
  void enableDefaultButton( bool bEnable )		{ enableWidget( defaultBtn, bEnable ); }
  void enableHelpButton( bool bEnable )		{ enableWidget( helpBtn, bEnable ); }
  void enableSUAButton( bool bEnable )		{ enableWidget( suaBtn, bEnable); }

  bool cancelButtonEnabled()			{ return widgetEnabled( m_lppbCancelBtn ); }
  bool applyButtonEnabled()			{ return widgetEnabled( m_lppbApplyBtn ); }
  bool okButtonEnabled()				{ return widgetEnabled( m_lppbOkBtn ); }
  bool defaultButtonEnabled()			{ return widgetEnabled( defaultBtn ); }
  bool helpButtonEnabled()			{ return widgetEnabled( helpBtn ); }
  bool suaButtonEnabled()			{ return widgetEnabled( suaBtn ); }

  void showCancelButton( bool bShow )		{ showWidget( m_lppbCancelBtn, bShow ); }
  void showApplyButton( bool bShow )		{ showWidget( m_lppbApplyBtn, bShow ); }
  void showOkButton( bool bShow )			{ showWidget( m_lppbOkBtn, bShow ); }
  void showDefaultButton( bool bShow )		{ showWidget( defaultBtn, bShow ); }
  void showHelpButton( bool bShow )		{ showWidget( helpBtn, bShow ); }
  void showSUAButton( bool bShow )		{ showWidget( suaBtn, bShow ); }

  bool cancelButtonVisible()			{ return widgetVisible( m_lppbCancelBtn ); }
  bool applyButtonVisible()			{ return widgetVisible( m_lppbApplyBtn ); }
  bool okButtonVisible()			{ return widgetVisible( m_lppbOkBtn ); }
  bool defaultButtonVisible()			{ return widgetVisible( defaultBtn ); }
  bool helpButtonVisible()			{ return widgetVisible( helpBtn ); }
  bool suaButtonVisible()			{ return widgetVisible( suaBtn ); }

  void enableAcceptance( bool bEnable );

protected:

  // helper methods	(AHMEDM)
  void enableWidget( QWidget* lpwToEnable, bool bEnable );
  bool widgetEnabled( QWidget* lpwEnabled );
  bool widgetVisible( QWidget* lpwEnabled );
  void showWidget( QWidget* lpwToShow, bool bShow );

  /// Resizes the dialog
  void resizeEvent(QResizeEvent *event);


protected slots:

  /// Closing the dialog will end the application
  virtual void done(int);

private:

  QPushButton *suaBtn;
  QPushButton *helpBtn;
  QPushButton *defaultBtn;

  QPushButton *m_lppbOkBtn;
  QPushButton *m_lppbApplyBtn;
  QPushButton *m_lppbCancelBtn;
};


/** The base widget for setup dialog.

    It provides methods to load and apply the settings.
*/
class KConfigWidget : public QWidget
{
  Q_OBJECT

public:

  /// Constructor.
  KConfigWidget(QWidget *parent, const char *name=0) : QWidget(parent, name) {};

  /// Loads the settings, usually from an rc-file.
  virtual void loadSettings() = 0;

  /// Applies the settings.
  virtual void applySettings() = 0;

  /// Sets default values.
  virtual void defaultSettings() {};
    
};


/** KControlApplication is the common base for setup applications.

    It provides a tab dialog and functionality common to most setup programs.

    @author Matthias H"olzer (hoelzer@physik.uni-wuerzburg.de)
    @short Common base for setup applications.
*/
class KControlApplication : public KApplication
{
  Q_OBJECT

public:

  /** Creates the setup application.

      The constructor scans the command line arguments. If there is a single argument, "-init",
      the function init() is called and the application terminates.
      Otherwise the setup dialog is created and inkoved.

      @param argc  number of commandline arguments
      @param argv  commandline arguments
      @param name  name of the application
   */
  KControlApplication(int &argc, char **argv, const char *name=0);

  /// Destructor. Cleans up.
  ~KControlApplication();

  /** Sets the title of the dialog.

      It's not possible to set the title within the constructor,
      because we need the application to get the translator and
      it would mean a lot of effort to do it without the one in kapp.

      @param title text to be shown in the dialogs titlebar
    */
  void setTitle(const char *title);

  /** Determines if the setup dialog has to be run.

      The setup dialog has to be run if the application has not been invoked with a single commandline
      argument containing "-init".

      Due to the fact the QApplication::exec() is not virtual, this construction has to be used to
      execute a KControlApplication:

      KControlApplication app(argc, argv, "name", "title");
      app.createPages();

      if (app.runGUI())
        return app.exec();
      else
        return 0;

      Just running app.exec() will fail if "-init" has been requested.
  */
  bool runGUI() { return !justInit; };


  /// Returns the tabbed dialog object.
  QTabDialog *getDialog() { return dialog; };


  /// Returns the list of pages to show.
  QStrList *getPageList() { return pages;};


  /// Adds a new page to the dialog.
  void addPage(QWidget *page, const QString &name, const QString &help_name);

  // returns a pointer to the OK button in the dialog
	QObject*	GetOKButton(QObject* pTabDialog);


public slots:
  /* The OK button has been rewired. Now, when OK is clicked, this function is
     called and checks for an unsuccessfull apply(). If apply*() failed, the dialog is
     not closed.
  */
	void OnOK();

  /** This function is called at startup to initialize the settings.
      This function must be overriden by all setup application that want to have persistent settings.
  */
  virtual void init() {};


  /** This function is called to apply the settings currently selected in the dialog.
      This function must be overriden by all setup applications.
  */
  virtual int apply() {return 0;};


  /** This function is called when the help button is pressed.
      The default behaviour is to call
      kapp->invokeHTMLHelp("kcontrol/$(appname)/$(help_name).html","");
  */
  virtual void help();


  /** This function is called when the user presses the default button.
      This function must be overriden by all setup application.
  */
  virtual void defaultValues() {};

  // BI: 242363 -- 1998/12/08 -- Sam Watts
  // Added supported for comm via stdin with KControl parent  
  void slotReceivedStdIn(int fdno);
  void slotProcessStdIn(char *buffer, int buflen);

  // This function is called when the Modify button is being pressed
  virtual void slotEnableModification() {};

protected:
  KControlDialog *dialog;
  QAccel         *controlAccel;
  QStrList       *pages;
  QStrList       helpNames;
  bool           justInit;

  int in[2];

  // BI: 242363 -- 1998/12/08 -- Sam Watts
  // Added supported for comm via stdin with KControl parent  
  QSocketNotifier *innot;

private:
  QString        swallowCaption;

signals:

  // BI: 242363 -- 1998/12/08 -- Sam Watts
  // Added supported for comm via stdin with KControl parent  
  void receivedStdIn(char *buffer, int buflen);

};

// BI: 242852 -- 1998/12/08 -- Sam Watts
// Incorpopate Super User access into appropriate control panels
class KAdminControlApplication : public KControlApplication
{
public:
  KAdminControlApplication(int &argc, char **argv, const char *name=0);
  ~KAdminControlApplication() {};

  virtual void slotEnableModification();

protected:
  virtual void enableAdminAccess(bool bEnable) {};
};

#endif
