#ifndef kiojob_h
#define kiojob_h

#include <kprocess.h> 
class KIODirectoryEntry;

#include <qobject.h>
#include "kstrlist.h"

class KIOJob : public QObject
{
    Q_OBJECT
public:
    enum Jobs { JOB_COPY, JOB_MOVE, JOB_DELETE, JOB_MOUNT, JOB_UNMOUNT, JOB_LIST, JOB_MKDIR, JOB_LINK, JOB_EJECT_CD, JOB_EMPTY_TRASH };
    
    /**
     * Constructs a new job.
     * '_id' can be used to identify the job. Every signal transmitts this id,
     * so the client always knows which job sent him the signal.
     */
    KIOJob( int _id = 0 );
    ~KIOJob();

    void displayErrors(bool bDisplayErrors) { m_bDisplayErrors = bDisplayErrors; };

		void copy( QStrList & _src_url_list, const char *_dest_dir_url );    
    void copy( const char *_src_url, const char *_dest_url );
    void move( QStrList & _src_url_list, const char *_dest_dir_url );    
    void move( const char *_src_url, const char *_dest_url );
    void link( QStrList & _src_url_list, const char *_dest_dir_url );    
    void link( const char *_src_url, const char *_dest_url );
    void del( QStrList & _url_list );    
    void del( const char *_url );
    /**
     * Mounts a device.
     * '_point' is NOT an URL, it is just an usual path like "/cdrom".
     * You must only specify '_dev' and set the others to 0L, or you must
     * give values for every parameter.
     */
    void mount( bool _ro, const char *_fstype, const char* _dev, const char *_point );
    void unmount( const char *_point );
    void ejectCD( const char *strDevice );

    /**
     * Gets a directory listing.
     * Connect to 'newDirEntry' to get the directory entries. You must expect that the
     * signal is emitte before this function returns ( if the entries are cached ) or
     * sometimes after the function returned.
     *
     * @param  _reload is TRUE if cached data should be deleted and the directory scanned again.
     * @param _bHTML tells wether we accept HTML code as a reesult. In this case
     *               the signal @ref #data will be emitted multiple times until
     *               the complete HTML code is transfered. For example the 'file'
     *               protocol sends HTML code if it finds some "index.html" file
     *               in the directory.
     */
    void list( const char *_url, bool _reload = FALSE, bool _bHTML = FALSE );
    void mkdir( const char *_url );    

    void emptyTrash();    

    /**
     * Deletes all running jobs and their slaves.
     * Do this only if you want to quit the application.
     */
    static void deleteAllJobs();

    /**
     * Allows the job to overwrite existing files during move and
     * copy commands.
     *
     * @see #overwriteExistingFiles
     */
    void setOverWriteExistingFiles( bool _o );

    /**
     * If this flag is set to TRUE, the instance deletes itself after
     * its job is finished. So you can start a KIOJob and dont have to care
     * about it any more. This flag is turned ON by default.
     */
    void setAutoDelete( bool _b ) { bAutoDelete = _b; }
    /**
     * @see #isAutoDelete
     */
    bool isAutoDelete() { return bAutoDelete; }

    /*
     * hack to get static classes up and running even with C++-Compilers/
     * Systems where a constructor of a class element declared static
     * would never get called (Aix, Alpha,...). In the next versions these
     * elements should disappear.
     */
    static void InitStatic()
    {
    	jobList = new QList<KIOJob>;
    	jobList->setAutoDelete( false );
    }

public slots:
	void cancel();

	void slotDirEntry( const char *_url, const char *_name, bool _isDir, int _size,
		       const char * _creationDate, const char * _access,
		       const char * _owner, const char *_group );

protected slots:
	void slotProcessExited(KProcess *);
	void slotReceivedStdout(KProcess *proc, char *buffer, int buflen);
	void slotReceivedStderr(KProcess *proc, char *buffer, int buflen);

signals:
	void data( const char *_data, int _len );
	void newDirEntry(const char* _name, const char* _access, const char* _owner,
									const char* _group,	const char* _date, int _size);

	/**
	* Emitted if the job has finished.
	*/
	void finished( int _id );

	/**
	* Emitted if a problem occured
	*/
	void error(int _kioerror, const char *_txt );

	/**
	* Tells about the amount of bytes read.
	* Not implemented yet.
	*/
	void bytes( int _bytes );

protected:  
    int doIt(const char *szCmd = 0);
		void submitParams(KProcess::Communication & nOut);
		
		void done();
		
    void copy();
    void move();
    void del();
    /**
     * This function does the real linking stuff on local file systems.
     */
    void link();
    
    KProcess *slave;

    /**
     * Used for COPY command.
     */
    KStrList cmSrcURLList;
    QString m_strDestDirURL;
    
    /**
     * Used to store infos for MOUNT / UNMOUNT command params.
     */
    QString mntFSType;
    QString mntDev;
    QString mntPoint;
    bool mntReadOnly;

    /**
     * The action that is currently taking place.
     */
    Jobs action;

    /**
     * A unique ID.
     */
    int m_nID;

    /**
     * List of all running jobs.
     */
    static QList<KIOJob> *jobList;

protected:
    /**
     * This flag is set to TRUE if existing files should be overwritten by copy/move jobs.
     */
    bool overwriteExistingFiles;
    
    /**
     * @see #setAutoDelete
     */
    bool bAutoDelete;

    /**
     * List of URLs to notify.
     * When a job is completed a notify signal is emitted for every URL in
     * this list.
     */
    KStrList notifyList;

		/*
		*	 When set to true error messages will be displayed
		*/
		bool m_bDisplayErrors;

private:
	QString completeURL( const char *_url);		

	QString m_strSav;
};

#endif
