// KIOServer
// Server side implementation

#ifndef kioserver_h
#define kioserver_h

#include <qobject.h>
#include <qlist.h>
#include <qdict.h>
#include <qstring.h>
#include <qtimer.h>
#include <qdatetime.h>

#include <kurl.h>

//@Man: KIO Modes
//@{
/** Modes used for the static 'supports' functions of the KIOs */
/// Read from the KIO
#define KIO_Read 1
/// Write to the KIO
#define KIO_Write 2
/// Make a directory
#define KIO_MakeDir 4
/// Delete a file
#define KIO_Delete 8
/// Link a file
#define KIO_Link 16
/// List directory entries
#define KIO_List 32
/// Moves files
#define KIO_Move 64
//@}

class KIOServer : public QObject
{
    Q_OBJECT

    KIOServer();

public:
    static KIOServer* getKIOServer();

    /**
     * @return the name that would be used if you want to copy
     *         the file. Usually this is like @ref KURL::filename,
     *         but for example "http://myhost/" would result in
     *         'http:myhost" for example
     */
    static QString getDestNameForCopy( const char * _url );
    /// Get a new name for a link.
    /**
      If the user wants to make a link to an URL we need to make a valid file
      name from the URL because the link needs a name.
      */
    static QString getDestNameForLink( const char * _url, const char * strDstDir = 0L);
    /// Tests wether the list of URLs are all together directories
    /**
      If the protocol of the URL is not "file", then this function
      depends an the convention, that directories have a trailing '/'
      in their URL.
      
      @return 1 if the URL is a directory, 0 if not and <0 if we are unshure.
      */
    static int isDir( QStrList & _urls );
    /// Tests wether _url is a directory or not
    /**
      If the protocol of the URL is not "file", then this function
      depends an the convention, that directories have a trailing '/'
      in their URL.

      @return 1 if the URL is a directory, 0 if not and <0 if we are unshure.
      */
    static int isDir( const char *_url );
    /**
     * Tests wether the list of URLs are all together the trash bin.
     * This function does not really make sense. It is just provided for
     * convenience.
     */
    static bool isTrash( QStrList & _urls );
    /**
     * Tests wether _url is the trash bin or not
     */
    static bool isTrash( const char *_url );
    static QString findDeviceMountPoint( const char *_device, const char *_file = "/etc/mtab" );
    /// Tells wether KIOServer supports an operation on an URL
    /**
      _mode is for example KIO_Read or any of those constants.
      */
    static bool supports( const char *_src_url, int _mode );
    /// Tells wether KIOServer supports an operation on some URLs
    /**
      An operation that is not supported by at least one of the URLs
      will cause this function to return FALSE.
      */
    static bool supports( QStrList & _urls, int _mode );

    /**
      This function may be called by others to emit a notify signal.
      */
    static void sendNotify( const char *_url );

    /**
      This function may be called by others to emit a mount notify signal.
      Every time a device is mounted unmounted, this signal is emitted. This
      causes some icons to change. A normal notify signal is emitted for
      the mount directory, too.
      */
    static void sendMountNotify();

    /**
     * Quotes a filename/URL like the shell ( bash ) would expect it.
     */
    static QString shellQuote( const char *_data );

    /**
     * Unquotes a filename/URL that is quoted like the shell ( bash ) would expect it.
     */
    static QString shellUnquote( const char *_data );

    /**
     * This function is used in @ref KfmView::slotDropEvent for example
     * to check wether some user tries to copy his home directory to his
     * Desktop, which is a sub directory of its home directory.
     *
     * @return true if '_src_url' does not contain '_dest_canonical'. 
     *
     * @param _src_url is a usual URL
     * @param _dest_canonical is a path on your local hadr drives that does not
     *                        contain any simlynks or a usual URL if the file this
     *                        URL points to is not a file on your local hard drives.
     */
    static bool testDirInclusion( const char *_src_url, const char *_dest_canonical );
    
    /**
     * Does nothing on protocols other then "file" or if there is a sub protocol
     * in the URL. The canonical URL does not contain any symlinks.
     */
    static QString canonicalURL( const char *_url );

signals:
    /**
      Connect to this signal if you need to know when to
      update your directory contents.
      */
    void notify( const char *_url );

    /**
    	Tells that some device has been mounted/unmounted.
      Connect to this signal to get the icons associated with the devices
      updated.
      */
    void mountNotify();

protected:
    /// Internal function used to implement 'sendNotify' as static.
    void sendNotify2( const char *_url );

    /// Internal function used to implement 'sendMountNotify' as static.
    void sendMountNotify2();

    // A pointer to the running server
    static KIOServer *m_pKIOServer;
};

#endif
